#include <stdio.h>
#include <stdlib.h>
#include <qdos.h>
#include <qptr.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <csrvthg.h>
#include <database.h>

extern char zipID[];
extern char lzhID[];
extern char arjID[];
extern char arcID[];

#define BMAX 42
extern char    unzip[BMAX], unlzh[BMAX], unarj[BMAX],
               unarc[BMAX], unhar[BMAX], unpcmd[BMAX];
extern char    inbound[BMAX];
extern char    store[BMAX];
extern char    logmsg[82];
extern int     line;
extern int     loglevel;

int     unpack    (char * fname);
long    getbuff   (char *s, long n, FILE *iop); /* custom read function */
int     copyfile  (char *s, char *t, char *f);  /* copy file */
int findref(long db, char *rec, struct fld_list fldef[],char *appcn);
int findanum(long db, char *rec, struct fld_list fldef[],char *appcn);
int findaname(long db, char *rec, struct fld_list fldef[],char *appcn);
void intel( long chan, int value);
void sysvar(char *varname, char *ans, int sz);
void log(int level, char *value);

/*---------------------------------------------------------------------------*/
int findref(long db, char *rec, struct fld_list fldef[],char *appcn)
/*
        search the database for field 3 = required number

long db;                             database ID
char *rec;                           pointer to record
struct fld_list fldef[];             field list
char *appcn;                         application definable quantity

   return: 1 = match;  0 = no match.
*/
{
   short  val;

   db = db; /* keep compiler happy */
   /* make a pointer to field */
   val = *(short *) (rec + fldef[3].fld_ptr);

   return( val == *(short *)appcn);
}
/*---------------------------------------------------------------------------*/
int findanum(long db, char *rec, struct fld_list fldef[],char *appcn)
/*        search the database for field 1 = required number */
{
   return(  *(short *)(rec + fldef[1].fld_ptr) == *(short *)appcn  );
}
int findaname(long db, char *rec, struct fld_list fldef[],char *appcn)
/*
        search the database for field 2 = required name

long db;                             database ID
char *rec;                           pointer to record
struct fld_list fldef[];             field list
char *appcn;                         application definable quantity; in
                                     this case, the compare string
   return: 1 = match;  0 = no match.
*/
{
   struct QLSTR *string;
   int stln;
 
   db = db; /* keep compiler happy */
   /* make a pointer to field 2 */
   string = (struct QLSTR *) (rec + fldef[2].fld_ptr);
   stln = strlen(appcn);
   if (stln != string->qs_strlen)  return(0);
   return(strnicmp(string->qs_str,appcn,stln) == 0);

}
/*---------------------------------------------------------------------------*/
void intel( long chan, int value)
{
  unsigned char *ptr;
  unsigned short word;
  unsigned char low,high;

  word = (unsigned short)value;
  ptr  = (unsigned char *)&word;
  high = *ptr;
  ptr++;
  low  = *ptr;

  (void)io_sbyte(chan, -1, low);
  (void)io_sbyte(chan, -1, high);
}

/*---------------------------------------------------------------------------*/
void sysvar(char *varname, char *ans, int sz)
{
char msg[80];
char *a;
short rlen;

  sprintf(msg,"sysvar %s",varname);
  a=Request ("pbox", msg, strlen(msg), &rlen);
  if (a == NULL)
    ans[0] = '\0';
  else
  {
    if (rlen < sz) sz = rlen;
    strncpy(ans,a,sz);
    ans[sz] = '\0';
    free(a);
  }
}
/*--------------------------------------------------------------------------*/
void log(int level,char *value)
{
char msg[80];
char *a;
short rlen;

 if((loglevel & level) || (level == 0))
 {
  sprintf(msg,"log %d %s",line,value);
  a=Request ("pbox", msg, strlen(msg), &rlen);
  if (a != NULL) free(a);
 }
}

/*---------------------------------------------------------------------------*/
long    getbuff(char *s, long n, FILE *iop)
{
   /* fetch at most n characters from iop or until EOF
      returning the number of characters read.          */

   int   c,f;

   for(f = 0; f < n-1  && (c = getc(iop)) != EOF; f++)
      s[f] = (char)c;
   s[f+1] = '\0';
   return f;
}
/*---------------------------------------------------------------------------*/
int copyfile(char *s, char *t, char *f)
{
        /*----------------------------------------------------------
            Copy file f (full name) from directory s to directory t
          ----------------------------------------------------------*/

FILE    *ip,*op;
int     c;
char    temp[44];

  c = strlen(s);                /* length of source_dir name on filename */

  (void)strcpy(temp,t);               /* get target directory */
  (void)strcat(temp,&f[c]);           /* append filename      */

  (void)strcpy(logmsg,"Copying ");
  (void)strcat(logmsg,f);
  (void)strcat(logmsg," to ");
  (void)strcat(logmsg,temp);
  log(8,logmsg);
  ip = fopen(f,"r");    /* open source */
  if(ip == NULL)
  {
    (void)strcpy(logmsg,"Error opening ");
    (void)strcat(logmsg,f);
    log(32,logmsg);
    return(-1);                 /* return if error */
  }
  op = fopen(temp,"w");         /* open new target */
  if(op == NULL)
  {
    (void)strcpy(logmsg,"Error opening ");
    (void)strcat(logmsg,temp);
    log(32,logmsg);
    (void)fclose(ip);           /* close input     */
    return(-1);                 /* return if error */
  }
  while ((c = fgetc(ip)) != EOF)
    (void)fputc(c,op);

  (void)fclose(ip);
  (void)fclose(op);

return(0);
}
/*---------------------------------------------------------------------------*/
int unpack(char * fname)
{
int match = 0;                  /* file type not matched */
long fail  = 0;                 /* file failed to unpack */
int temp  = 0;
int err   = 0;
char buffer[12];                /* buffer to read file start into */
FILE *fp;                       /* file pointer for source file   */
char cmdline[102];              /* command line for unpackers     */
int   chantab[3] = { 2, 0, 0 }; /* channel table for Jans unarc   */

    /* make the filename lower case to aid pattern matching */
    while (fname[temp] != '\0')
    {
      fname[temp]=(char)tolower(fname[temp]);
      temp++;
    }

   fp = fopen(fname,"r");
   if (fp == NULL)
      {
      (void)strcpy(logmsg,"Cannot open \"");
      (void)strcat(logmsg,fname);
      (void)strcat(logmsg,"\"");
      log(32,logmsg);
      return(-1);
      }

   /* fetch 8 bytes from the file into a buffer allowing room for terminator */
   temp = getbuff(buffer, 9, fp);   /* custom function returns 0 on error  */
   if (temp == 0)
      {
      (void)strcpy(logmsg,"Error reading ");
      (void)strcat(logmsg,fname);
      log(32,logmsg);
      return(-1);
      }

    (void)fclose(fp);

   /* Now check the header */

   err = strncmp(buffer, zipID, strlen(zipID));
   if (err == 0)
   {
      (void)strcpy(logmsg,"Unzip ");
      (void)strcat(logmsg,fname);
      log(8,logmsg);
      sysvar("unzip_cmd",unpcmd,BMAX);
      (void)sprintf(cmdline,unpcmd,fname,inbound);
      fail =  execl( unzip, (int *)-1L, "Unzip", cmdline, NULL);
      if(fail != 0)
      {
        (void)strcpy(logmsg,"Failed to unzip ");
        (void)strcat(logmsg,fname);
        log(32,logmsg);
                                        /* if not unzipped print warning */
        if(strlen(store))               /* and if we were given a store, */
        {
          err = copyfile(inbound,store,fname); /* copy the file into it  */
          if(err == 0)
            (void)remove(fname);
        }
      }
      else
      (void)remove(fname);
      match = 1;
   }
   if(!match)
   {
   char a,b,c;
   int ext;
   a = buffer[0];       /* fudge for lzh which has variable bytes 0 & 1 */
   b = buffer[1];       /* so we fix these to known values and then     */
   buffer[0] = 1;       /* replace them later                           */
   buffer[1] = 1;
   err = strncmp(buffer, lzhID, strlen(lzhID));
   buffer[0] = a;
   buffer[1] = b;
   if (err == 0)
      {
      int offset;                       /* pointer to filename extension */
      offset = strfnd("_hrq",fname, 0); /* look for '_hrq' */
      if(offset == -1)
        offset = strfnd(".hrq",fname, 0); /* look for '.hrq' instead */
      if(offset == -1)
      {
         (void)strcpy(logmsg,"Unlzh ");
         (void)strcat(logmsg,fname);
         log(8,logmsg);
         sysvar("unlzh_cmd",unpcmd,BMAX);
         (void)sprintf(cmdline,unpcmd,fname,inbound);
         fail =  execl( unlzh, (int *)-1L, "Unlzh", cmdline, NULL);
         if(fail != 0)
         {
           (void)strcpy(logmsg,"Failed to unlzh ");
           (void)strcat(logmsg,fname);
           log(32,logmsg);
                                           /* if not unzipped print warning */
           if(strlen(store))               /* and if we were given a store, */
           {
             err = copyfile(inbound,store,fname); /* copy the file into it  */
             if(err == 0)
               (void)remove(fname);
           }
         }
         else
         (void)remove(fname);          /* and finally delete the zip    */
         match = 1;
      }
      else
      {
         (void)strcpy(logmsg,"Unhar ");
         (void)strcat(logmsg,fname);
         log(8,logmsg);
         ext=strlen(fname);                 /* har needs the extension off */
         c=fname[ext-4];
         fname[ext-4] = '\0';
         sysvar("unhar_cmd",unpcmd,BMAX);
         (void)sprintf(cmdline,unpcmd,fname,inbound);
         fname[ext-4]=c;
         fail =  execl( unhar, (int *)-1L, "Unhar", cmdline, NULL);
         if(fail != 0)
         {
           (void)strcpy(logmsg,"Failed to unhar ");
           (void)strcat(logmsg,fname);
           log(32,logmsg);
                                           /* if not unzipped print warning */
           if(strlen(store))               /* and if we were given a store, */
           {
             err = copyfile(inbound,store,fname); /* copy the file into it  */
             if(err == 0)
               (void)remove(fname);
           }
         }
         else
         (void)remove(fname);          /* and finally delete the zip    */
         match = 1;
      }
      match = 1;
      }
   }
   if(!match)
   {
     err = strncmp(buffer, arjID, strlen(arjID));
     if (err == 0)
     {
        (void)strcpy(logmsg,"Unarj ");
        (void)strcat(logmsg,fname);
        log(8,logmsg);
         sysvar("unarj_cmd",unpcmd,BMAX);
         (void)sprintf(cmdline,unpcmd,fname,inbound);
        fail =  execl( unarj, (int *)-1L, "Unarj", cmdline, NULL);
        if(fail != 0)
        {
          (void)strcpy(logmsg,"Failed to unarj ");
          (void)strcat(logmsg,fname);
          log(32,logmsg);
                                          /* if not unzipped print warning */
          if(strlen(store))               /* and if we were given a store, */
           {
             err = copyfile(inbound,store,fname); /* copy the file into it  */
             if(err == 0)
               (void)remove(fname);
           }
        }
         else
        (void)remove(fname);          /* and finally delete the zip    */
        match = 1;
     }
   }
   if(!match)
   {
     err = strncmp(buffer, arcID, strlen(arcID));
     if (err == 0)
     {
        (void)strcpy(logmsg,"Unarc ");
        (void)strcat(logmsg,fname);
        log(8,logmsg);
         sysvar("unarc_cmd",unpcmd,BMAX);
         (void)sprintf(cmdline,unpcmd,fname,inbound);
              /*
                  pass Jan's unarc the source dir too for unpacking into
                 Jan's UNARC needs to be passed only 1 open channel for output
              */

        chantab[0]=1;
        chantab[1]=fileno(stdout);
        fail =  execl( unarc, (int *)chantab, "Unarc", cmdline, NULL);
        if(fail != 0)
        {
          (void)strcpy(logmsg,"Failed to unarc ");
          (void)strcat(logmsg,fname);
          log(32,logmsg);
                                          /* if not unzipped print warning */
          if(strlen(store))               /* and if we were given a store, */
           {
             err = copyfile(inbound,store,fname); /* copy the file into it  */
             if(err == 0)
               (void)remove(fname);
           }
        }
         else
           (void)remove(fname);
        match = 1;
     }
   }
   if(!match)
   {
      (void)strcpy(logmsg,"Unknown archive type: ");
      (void)strcat(logmsg,fname);
      log(32,logmsg);
   }
   return fail;
}
